// Copyright (c) 2017 Douglas Miller <durgadas311@gmail.com>
import java.util.Properties;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

// Add-on Card Layout (GridBagConstraints):
// (when layout() is called)
//                                                             
//        |              (840 pixels total)                  |
//      |0|1|   2     |   3     |   4    |         5         |6|
//    - +------------------------------------------------------+--
//  0   |X| |         |         |        |                   |X|
//    - |X|-|- - - - -|- - - - -|- - - - | - - - - - - - - - |X|
//  1   |X                                                    X|
//      |X| |         |         |        |                   |X|
//    - |X - - - - - - - - - - - - - - - - - - - - - - - - -  X| (427 pixels total)
//      |X| |         |         |        |                   |X|
//  2   |X                                                    X|
//      |X| |         |         |        |                   |X|
//    - |X - - - - - - - - - - - - - - - - - - - - - - - - - -X| --
//  3   |XXXXXXXXXXXXX|XXXXXXXXX|XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX| (5 pixels high)
//    - +---+ - - - - - - - - - - - - -  +---------------------+ --
//  4    XXX|XXXXXXXXX|XXXXXXXXX|XXXXXXXX|XXXXXXXXXXXXXXXXXXXXX (53 pixels high)
//    - - - +----------------------------+ - - - - - - - - - - - --
//
// X = reserved cells/regions.
//
// Cells (1,0) through (5,2) may be used, within constraints of
// prescribed widths and total height.
//
// Column widths:
//    0 =  10 pixels (reserved)
//    1 =  52 pixels
//    2 = 129 pixels
//    3 = 139 pixels
//    4 = 140 pixels
//    5 = 380 pixels
//    6 =  10 pixels (reserved)
//
// Columns 0-2 height is fluid, within the total 427 pixels.
//
// Typical usage, standard H89 add-on card:
//
// class XXXX extends AddOnCard ...
//	...
//	layoutCard();
//	connectorListener(this);
//	...add components...
//		place(comp, x, y, dx, dy);
//	finish();
//
//	...actionPerformed...
//		if (connectorPerformed(e)) {
//			...clicked connector... (setVisible(false))
//
// -----------------------------
// Custom card usage (e.g. Z19, uses AddOnCard JFrame directly):
//
// class XXXX extends AddOnCard ...
//	...
//	getContentPane().setBackground(AddOnCard.PCB);
//	init(this);
//	...add components...
//		place(comp, x, y, dx, dy);
//	pack();
//

public abstract class AddOnCard extends JFrame {
	// 'PCB' may be changed and takes affect everywhere.
	public static final Color PCB = new Color(122, 171, 99);
	// 'PLUG' is matched to color in PNGs, so should not be changed.
	public static final Color PLUG = new Color(147, 35, 13);
	protected GridBagLayout gb = null;
	protected GridBagConstraints gc;
	protected Container top;
	private JButton p601 = null;
	private JButton p602 = null;

	public AddOnCard(String title) {
		super(title);
	}
	abstract String getCardName();
	abstract void setProperties(Properties props);
	abstract void updateProperties(Properties props);
	abstract void removeProperties(Properties props);
	abstract boolean configure(Component top);

	// caller may customize GridBagConstraints later...
	protected void init(Container top) {
		if (gb != null) {
			return;
		}
		gb = new GridBagLayout();
		gc = new GridBagConstraints();
		gc.fill = GridBagConstraints.NONE;
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 1;
		gc.weighty = 1;
		gc.gridwidth = 1;
		gc.gridheight = 1;
		gc.insets.top = 0;
		gc.insets.bottom = 0;
		gc.insets.left = 0;
		gc.insets.right = 0;
		gc.anchor = GridBagConstraints.CENTER;
		this.top = top;
		top.setLayout(gb);
	}

	protected void layoutCard() {
		JPanel pn;
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(860, 485));
		pn.setBackground(PCB);
		pn.setOpaque(true);
		init(pn);

		// side margins
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(10, 432));
		pn.setOpaque(false);
		place(pn, 0, 0, 1, 4);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(10, 432));
		pn.setOpaque(false);
		place(pn, 6, 0, 1, 4);

		// space enforcement
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(52, 5));
		pn.setOpaque(false);
		place(pn, 1, 3, 1, 1);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(129, 5));
		pn.setOpaque(false);
		place(pn, 2, 3, 1, 1);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(139, 5));
		pn.setOpaque(false);
		place(pn, 3, 3, 1, 1);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(140, 5));
		pn.setOpaque(false);
		place(pn, 4, 3, 1, 1);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(380, 5));
		pn.setOpaque(false);
		place(pn, 5, 3, 1, 1);

		// cut-outs
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(62, 53));
		pn.setBackground(Color.gray);
		place(pn, 0, 4, 2, 1);
		pn = new JPanel();
		pn.setPreferredSize(new Dimension(390, 53));
		pn.setBackground(Color.gray);
		place(pn, 5, 4, 2, 1);

		// host bus connectors
		gc.anchor = GridBagConstraints.SOUTH;
		gc.insets.left = 5;
		gc.insets.right = 2;

		p601 = new JButton();
		p601.setBorder(null);
		p601.setBackground(PLUG);
		p601.setPreferredSize(new Dimension(122, 26));
		place(p601, 2, 4, 1, 1);
		gc.insets.left = 3;
		gc.insets.right = 5;
		p602 = new JButton();
		p602.setBorder(null);
		p602.setBackground(PLUG);
		p602.setPreferredSize(new Dimension(271, 26));
		place(p602, 3, 4, 2, 1);
		gc.insets.left = 0;
		gc.insets.right = 0;
		gc.anchor = GridBagConstraints.CENTER;
	}

	protected void place(JComponent pn, int x, int y, int dx, int dy) {
		gc.gridx = x;
		gc.gridy = y;
		gc.gridwidth = dx;
		gc.gridheight = dy;
		gb.setConstraints(pn, gc);
		top.add(pn);
	}

	protected void finish() {
		if (top == this) {
			return;
		}
		add(top);
		pack();
	}

	protected void connectorListener(ActionListener lstn) {
		if (p601 == null) {
			return;
		}
		p601.addActionListener(lstn);
		p602.addActionListener(lstn);
	}

	protected boolean connectorPerformed(ActionEvent e) {
		return (e.getSource() == p601 || e.getSource() == p602);
	}
}
